--[[

Idea: Provide filter / replacement service for commonly used acronymns in game.
	links when moused over in default chat frames should show the game tooltip
	with the description provided.

Addon Name: AcronymGuide (AG)


]]--

local rev = "$Rev: 37 $"
local distro = "r41"

local function Debug(self, ...)
	local str = string.join(", ", tostringall(...) )
	str = str:gsub(":,", ":"):gsub("=,", "=")
	--@alpha@
	if true then
		ChatFrame5:AddMessage("|cff6666ffAcronymGuide:|r "..str)
	end
	--@end-alpha@
	return str
end

local tinsert, select, assert, tostringall = table.insert, select, assert, tostringall

AcronymGuide = {}
local addon = AcronymGuide
addon.rev = rev
addon.distro = distro

local frame = CreateFrame("frame")
frame:SetScript("OnEvent", function(frame, event, ...)
	if type(addon[event]) == "function" then
		addon[event](addon, event, ...)
	end
end)
frame:RegisterEvent("PLAYER_LOGIN")

addon.frame = frame
addon.Debug = Debug
addon.PlayerLoginFunc = {}

local localizedName = "|cff6666ffAcronymGuide:|r"	--L["AcronymGuide"]

function addon:PLAYER_LOGIN(event)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", self.DA_Main_Filter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", self.DA_Main_Filter)

	self:HookTooltip(GameTooltip)
	self:HookTooltip(ItemRefTooltip)
	for i = 1, #self.PlayerLoginFunc do
		self.PlayerLoginFunc[i](self)
	end
	self:Debug("OnEnable")
end

--------------------
--	Link Registry Stuff
--------------------

local key_link, key_desc = {}, {}
addon.key_link = key_link
addon.key_desc = key_desc
function addon:AddDefinition_A(txt)	--key = string, desc = table
	local key, linkText, desc = tostringall(string.match(txt, "^(%S+)|(%S+)|(.-)$"))

	assert( (key ~= "nil") or (key ~="" ))
	assert( (linkText ~= "nil") or (linkText ~="" ) )
	assert( (desc ~= "nil") or (desc ~="" ) )

	local link = self:GetLink(key, linkText)
	key_link[key] = link
	key_desc[key] = desc
	return true
end


local Master_Link_Format = "|cff6666ff|HAG:%s~|h[%s]|h|r"
local reverse_regex = "AG:(.-)~"

addon.Master_Link_Format = Master_Link_Format
addon.Unknown_Link = Unknown_Link
addon.reverse_regex = reverse_regex


function addon:GetLink(uid, linkText)
	assert( (uid) and ( type(uid) == "string") )
	if not linkText then
		if key_link[uid] then
			return key_link[uid]
		else
			return uid
		end
	end

	return Master_Link_Format:format(uid, linkText)
end


--------------------
---	Filter Stuff
--------------------
--[[
	Using the method from WIM's Emoticons.lua file where it replaces the links with a \001\004 + table index.
	i believe i can expand on that by just replacing the things i wants links for wtih the same method and adding to said table.
]]--

do
	local tempLinks = {}

	local function existingLinkTrade(theLink)
		tinsert(tempLinks, theLink);
		return "\001\004"..#tempLinks;
	end

	local function customLink_MethodC(ele)
		ele = ele:lower():trim()
		if key_link[ele] then
			tinsert(tempLinks, key_link[ele])
			return "\001\004"..#tempLinks;
		end
	end

	function addon.DA_Main_Filter(frame, event, daMsg, ...)
		if (daMsg == "nil") or (daMsg == "") then
			return false, daMsg, ...
		end

		--Remove Existing Links
		wipe(tempLinks)
		local result
		repeat
			daMsg, result = string.gsub(daMsg, "(|c%x+|H[^|]+|h[^|]+|h|r)", existingLinkTrade , 1);
		until result == 0;	--End

		-- Our Link Method
		daMsg = daMsg:gsub("%f[%w\192-\255]([%w\128-\255]+)%f[^%w\128-\255]", customLink_MethodC )

		--Cleanup
		--Put the existing links back
		for i=1, #tempLinks do
			daMsg = daMsg:gsub("\001\004"..i, tempLinks[i] )
		end
		wipe(tempLinks)

		return false, daMsg, ...
	end

end


--------------------
---	Tooltip Hooks
--------------------

--[[
	Make a generator function that other addons can use.
	hook the chatframes so that we have hoverlinks.	--gonna grab it from tekKompare, if your going to steal, steal from the best :)
]]--
do
	local function colorCapLetterGreen(str)
		if str == str:upper() then
			return "|cff1eff00"..str:upper().."|r"
		else
			return str
		end
	end

	local function addMultiLine( tip , ele, nx, ...)
		if not ele then
			return
		elseif ele == 1 then
			tip:AddLine( (nx:gsub(".", colorCapLetterGreen)) , nil, nil, nil, 1)
			return addMultiLine(tip, ...)
		else
			tip:AddLine(ele, nil, nil, nil, 1)
			return addMultiLine(tip, nx, ...)
		end
	end

	local hooked_tooltips = {}

	local function new_SetHyperlink(tip, id, txt, ...)
		if not hooked_tooltips[tip] then
			print("AG Tooltip Hooked called for", tip.GetName and tip.GetName() or tip, "and is not registered.", debugstack() )
		end
		if id:find(reverse_regex) then
			local _, _, uid = id:find(reverse_regex)
			--if tip.AcronymGuideID and tip.AcronymGuideID == id then
			--	--If we're parented to a chat frame then don't close.
			--	if tip:IsShown() and addon.chatFrame_onEnter[tip:GetOwner()] then return end
			--	tip.AcronymGuideID = nil
			--	tip:Hide()
			--	return
			--end
			tip:ClearLines()
			tip:AddLine(localizedName)
			local desc = key_desc[uid] and key_desc[uid] or ""
			if desc:find("\002") then
				addMultiLine(tip, 1, string.split("\002", desc) )
			else
				tip:AddLine((desc:gsub(".", colorCapLetterGreen)), nil, nil, nil, 1)
			end
			tip.AcronymGuideID = id
			tip:Show()
			return
		end

		return hooked_tooltips[tip](tip, id, txt, ...)
	end


	function addon:HookTooltip(tip)			--Generator Function for 3rd party tooltips...
		if hooked_tooltips[tip] then
			return true
		end
		hooked_tooltips[tip] = tip.SetHyperlink
		tip.SetHyperlink = new_SetHyperlink
	end
end

do	--Straight Copy and Paste from tekKompare\Hovertips.lua by Tekkub
	local cf_onEnter, cf_onLeave = {}, {}
	addon.chatFrame_onEnter, addon.chatFrame_onLeave = cf_onEnter, cf_onLeave
	local GameTooltip = GameTooltip

	local linktypes = { AG = true }	--Only Line Modified, adapted to specific needs of Addon

	local function OnHyperlinkEnter(frame, link, ...)
		local linktype = link:match("^([^:]+)")
		if linktype and linktypes[linktype] then
			GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetHyperlink(link)
			GameTooltip:Show()
		end

		if cf_onEnter[frame] then return cf_onEnter[frame](frame, link, ...) end
	end

	local function OnHyperlinkLeave(frame, ...)
		GameTooltip:Hide()
		if cf_onLeave[frame] then return cf_onLeave[frame](frame, ...) end
	end
	local noop = function() end
	local _G = getfenv(0)
	for i=1, NUM_CHAT_WINDOWS do
		local frame = _G["ChatFrame"..i]
		cf_onEnter[frame] = frame:GetScript("OnHyperlinkEnter") or noop
		frame:SetScript("OnHyperlinkEnter", OnHyperlinkEnter)

		cf_onLeave[frame] = frame:GetScript("OnHyperlinkLeave") or noop
		frame:SetScript("OnHyperlinkLeave", OnHyperlinkLeave)
	end
end
